// Amaris Vanegas
#include <Arduino.h>
#include <esp_now.h>
#include <WiFi.h>
#include <../lib/HTTPClient/src/HTTPClient.h>
#include <ArduinoJson.h>
#include <typeinfo>

// Replace with your network credentials
const char* ssid = "SDNet";
const char* password = "CapstoneProject";
 
// Replace with your Back4App application keys
const char* parseAppId = "JJsjSHIsybIqlWDUU7QeY8AALLsjIr3f8YWWvNNV";
const char* restApiKey = "LmmOY7jQ8BhJzh0FhehZGtsVQjI4VNkfzhzQAQmR";

// Create an HTTPClient object 
WiFiClient client;
HTTPClient http;

// Basic URL for the API requests
String basic_url = "https://parseapi.back4app.com/classes/DeviceReadings";

// Structure example to receive data
// Must match the sender structure
typedef struct struct_message {
  int id;
  int x;
  int y;
}struct_message;

// Create a struct_message called myData
struct_message myData;

// Create a structure to hold the readings from each board
struct_message board1;
struct_message board2;
struct_message board3;
struct_message board4;

// Create an array with all the structures
struct_message boardsStruct[4] = {board1, board2, board3, board4};

// callback function that will be executed when data is received
void OnDataRecv(const uint8_t * mac_addr, const uint8_t *incomingData, int len) {
  char macStr[18];
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  memcpy(&myData, incomingData, sizeof(myData));
  Serial.printf("Board ID %u: %u bytes\n", myData.id, len); //replace with if else statement that prints sensor name depending on id
  // Update the structures with the new incoming data
  boardsStruct[myData.id-1].x = myData.x;
  boardsStruct[myData.id-1].y = myData.y;
  Serial.printf("x value: %d \n", boardsStruct[myData.id-1].x);
  Serial.printf("y value: %d \n", boardsStruct[myData.id-1].y);
  Serial.println();
}
 
void setup() {
  //Initialize Serial Monitor
  Serial.begin(115200);
  
  //Set device as a Wi-Fi Station
  WiFi.mode(WIFI_STA);

  // Connect to Wi-Fi
  Serial.println("Connecting to Wi-Fi...");
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting...");
  }
  Serial.println("Connected to Wi-Fi!");

  //Init ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  
  // Once ESPNow is successfully Init, we will register for recv CB to
  // get recv packer info
  esp_now_register_recv_cb(OnDataRecv);
}
 
void dataFunc(int i){
  int boardX = boardsStruct[i].x;
  int boardY = boardsStruct[i].y;
  String stringX = String(boardX);
  String stringY = String(boardY);
  String deviceType = "";
  // URL complement for the GET request for the fourth board
  if (i == 0){
    deviceType = "GasSensor";
  }else if(i == 1){
    deviceType = "PIRMotion";
  }else if(i == 2){
    deviceType = "ContactSensor";
  }else if(i == 3){
    deviceType = "Accelerometer";
  }
  String get_complement = "?where={\"DeviceType\":\""+ deviceType + "\",\"DeviceCode\":\"" + stringY + "\"}";
  String get_url = basic_url + get_complement;
  Serial.println(get_url);
  //begin and keys
  http.begin(get_url);
  http.addHeader("X-Parse-Application-Id", parseAppId);
  http.addHeader("X-Parse-REST-API-Key", restApiKey);
  int httpGetRetrieve = http.GET(); 
  // If statement 
  if (httpGetRetrieve == HTTP_CODE_OK) {
    // Parse the JSON response
    String payload = http.getString();
    DynamicJsonDocument doc(1024);
    DeserializationError error = deserializeJson(doc, payload);

    // Extract the row(s) of data you are interested in
    JsonArray results = doc["results"];
    String value = results[0]["objectId"];
    Serial.println("Results:");
    Serial.println(results);

    if (value[0] == 'n' && value[1] == 'u' && value[2] == 'l' && value[3] == 'l'){
      Serial.println("Chegou aqui!");
      http.end();
      http.begin(basic_url);
      http.addHeader("Content-Type", "application/json");
      http.addHeader("X-Parse-Application-Id", parseAppId);
      http.addHeader("X-Parse-REST-API-Key", restApiKey);
      String data = "{\"DeviceType\":\""+ deviceType + "\",\"DeviceCode\":\"" + stringY + "\",\"Reading\":\"" + stringX +"\"}";
      Serial.println(data);
      int httpResponseCode = http.POST(data);
      if (httpResponseCode == 201){
        Serial.println("POST Successful");
      }
      else{
        Serial.println("POST Failed");
      }
      http.end();
    }
    else{
      String put_url = basic_url + "/" + value;
      Serial.println(put_url);
      String data = "{\"DeviceType\":\""+ deviceType + "\",\"DeviceCode\":\"" + stringY + "\",\"Reading\":\"" + stringX +"\"}";
      Serial.println(data);
      http.end();
      http.begin(put_url);
      http.addHeader("Content-Type", "application/json");
      http.addHeader("X-Parse-Application-Id", parseAppId);
      http.addHeader("X-Parse-REST-API-Key", restApiKey);
      int httpResponseCode = http.PUT(data);
      if (httpResponseCode == 200){
        Serial.println("PUT Successful");
      }
      else{
        Serial.println("PUT Failed");
      }
      http.end();
    }
  }
  else{
    Serial.println("GET Failed");
  }

}

void loop() {
  dataFunc(0);
  dataFunc(1);
  dataFunc(2);
  dataFunc(3);
  delay(1000);
}